@@ -407,6 +407,58 @@ class TestListVulnerabilityTemplateView(ReadWriteAPITests, BulkUpdateTestsMixin,
         vuln_template = session.query(VulnerabilityTemplate).filter_by(id=template.id).first()
         assert vuln_template.custom_fields == {'cvss': 'updated value'}
 
+    def test_create_vuln_template_with_cvss4(self, session, test_client):
+        raw_data = {
+            "cwe": "",
+            "description": "test description",
+            "desc": "test description",
+            "exploitation": "high",
+            "name": "test template",
+            "references": [],
+            "refs": [],
+            "resolution": "test resolution",
+            "type": "vulnerability_template",
+            "cvss4": {
+                "vector_string": "CVSS:4.0/AV:N/AC:L/AT:N/PR:N/UI:N/VC:H/VI:H/VA:H/SC:N/SI:N/SA:N"
+            }
+        }
+
+        res = test_client.post(self.url(), data=raw_data)
+        assert res.status_code == 201
+        assert 'cvss4' in res.json
+        assert res.json['cvss4']['vector_string'] == raw_data['cvss4']['vector_string']
+
+        vuln_template = session.query(VulnerabilityTemplate).filter_by(id=res.json['_id']).first()
+        assert vuln_template.cvss4_vector_string == raw_data['cvss4']['vector_string']
+
+    def test_update_vuln_template_with_cvss4(self, session, test_client):
+        template = VulnerabilityTemplateFactory.create()
+        session.add(template)
+        session.commit()
+
+        raw_data = {
+            "cwe": "",
+            "description": "updated description",
+            "desc": "updated description",
+            "exploitation": "critical",
+            "name": "updated template",
+            "references": [],
+            "refs": [],
+            "resolution": "updated resolution",
+            "type": "vulnerability_template",
+            "cvss4": {
+                "vector_string": "CVSS:4.0/AV:L/AC:H/AT:P/PR:L/UI:R/VC:L/VI:L/VA:L/SC:L/SI:L/SA:L"
+            }
+        }
+
+        res = test_client.put(self.url(template.id), data=raw_data)
+        assert res.status_code == 200
+        assert 'cvss4' in res.json
+        assert res.json['cvss4']['vector_string'] == raw_data['cvss4']['vector_string']
+
+        vuln_template = session.query(VulnerabilityTemplate).filter_by(id=template.id).first()
+        assert vuln_template.cvss4_vector_string == raw_data['cvss4']['vector_string']
+
     def test_add_vuln_template_from_csv(self, session, test_client, csrf_token):
         expected_created_vuln_template = 1
         vuln_template_name = "EN-Improper Restriction of Operations within the Bounds of a Memory Buffer (Type: Class)"
